/*
 *	Copyright (C) 2020 3D Repo Ltd
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Affero General Public License as
 *	published by the Free Software Foundation, either version 3 of the
 *	License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Affero General Public License for more details.
 *
 *	You should have received a copy of the GNU Affero General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "CoreMinimal.h"
#include "Templates/SharedPointer.h"

/*
 * The SampleWebRequestHelpers class contains convenience methods for accessing the 3D Repo Web API.
 * Full documentation is available at https://3drepo.github.io/3drepo.io/
 * This class is illustrative and is not meant as a comprehensive mirror of the Web API.
 * If an API is not reflected here, it is up to the user code to implement it, e.g. using the Unreal 
 * JSON serialisation and the RepoWebRequestManager.
 * 
 * Note that to build SampleWebRequestHelpers, you must add the Json and JsonUtilities to the 
 * .Build.cs PrivateDependencies list:
 *		e.g. PrivateDependencyModuleNames.AddRange(new string[] { "Repo3d", "Json", "JsonUtilities" });
 */
class SampleWebRequestHelpers
{
public:
	class Tree
	{
	public:
		class Node
		{
		public:
			FString account;
			FString project;
			FString type;
			FString name;
			FString path;
			FString _id;
			FString shared_id;
			bool toggleState;
			TArray<TSharedRef<Node>> children;
		};

		TSharedRef<Node> nodes;
		TMap<FString, FString> idToName;

		Tree() :nodes(MakeShared<Node>())
		{
			nodes->_id = "";
		}
	};

	DECLARE_DELEGATE_OneParam(GetTreeDelegate, TSharedRef<Tree>);
	static void GetTree(TSharedRef<class RepoWebRequestManager> manager, FString teamspace, FString model, FString revision, GetTreeDelegate callback);
};