/*
 *	Copyright (C) 2020 3D Repo Ltd
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Affero General Public License as
 *	published by the Free Software Foundation, either version 3 of the
 *	License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Affero General Public License for more details.
 *
 *	You should have received a copy of the GNU Affero General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "CoreMinimal.h"
#include "GameFramework/GameModeBase.h"
#include "UObject/WeakInterfacePtr.h"
#include "repounrealGameModeBase.generated.h"

/*
 * Contains information about a mesh selection event
 */
struct MeshIdSelection
{
	bool Valid;
	FString Id;
	FVector WorldLocation;
	TWeakInterfacePtr<class IRepoTraceable> Traceable;
};

/*
 * This PlayerController implements some additional example functionality, including mesh selection and changing of mesh properties.
 * The PlayerController is set as the default Controller for the project in the settings INI.
 * For your own application, create a new PlayerController subclass and set it as the project's default.
 * 
 * This PlayerController binds an action to a mouse click. When the model is clicked (in Play Mode), the mesh Id under the cursor is
 * retrieved, printed to the console, and the colour of the object set to white.
 */
UCLASS()
class REPOUNREAL_API ArepounrealPlayerController : public APlayerController
{
	GENERATED_BODY()

public:
	ArepounrealPlayerController();

	void BeginPlay();
	MeshIdSelection GetMeshIdUnderCursor();
	void OnMouseClick();
};

/**
 * 
 */
UCLASS()
class REPOUNREAL_API ArepounrealGameModeBase : public AGameModeBase
{
	GENERATED_BODY()

public:
	ArepounrealGameModeBase();
};
